

#ifndef __ML16P_DRIVER_H
#define __ML16P_DRIVER_H




#define  ML16P_VERSION  "Linux ML16-P driver 1.0.4, 96 Jul 30"
#define  ML16P_ID       "ml16p: "


#ifndef  ML16P_MAX_CARDS
   #define  ML16P_MAX_CARDS  (2)      /*  Max number of cards to probe for.  */
#endif


#define  ML16P_MAJOR  (39)            /*  The device major number.           */


#define  ML16P_PROBE_IO_START  0x100  /*  Lowest I/O port to probe.          */
#define  ML16P_PROBE_IO_END    0x3b0  /*  Highest I/O port.                  */

#define  ML16P_PROBE_IRQ_START  (2)   /*  Lowest IRQ to look for cards.      */
#define  ML16P_PROBE_IRQ_END    (7)   /*  Highest IRQ to look for cards.     */

#define  ML16P_MINOR_LARGEST  (21)   /*  The largest minor number used.      */
#define  ML16P_MINOR_RANGE    (32)   /*  Number of minor numbers allocated   */
                                     /*  per card.                           */
#define  ML16P_MINOR_ADC0     (0)    /*  These minor numbers are added to    */
#define  ML16P_MINOR_ADC1     (1)    /*  the appropriate base minor number   */
#define  ML16P_MINOR_ADC2     (2)    /*  of a card to find the actual minor  */
#define  ML16P_MINOR_ADC3     (3)    /*  minor number.                       */
#define  ML16P_MINOR_ADC4     (4)    /*                                      */
#define  ML16P_MINOR_ADC5     (5)    /*                                      */
#define  ML16P_MINOR_ADC6     (6)    /*                                      */
#define  ML16P_MINOR_ADC7     (7)    /*                                      */
#define  ML16P_MINOR_ADC8     (8)    /*                                      */
#define  ML16P_MINOR_ADC9     (9)    /*                                      */
#define  ML16P_MINOR_ADC10    (10)   /*                                      */
#define  ML16P_MINOR_ADC11    (11)   /*                                      */
#define  ML16P_MINOR_ADC12    (12)   /*                                      */
#define  ML16P_MINOR_ADC13    (13)   /*                                      */
#define  ML16P_MINOR_ADC14    (14)   /*                                      */
#define  ML16P_MINOR_ADC15    (15)   /*                                      */
#define  ML16P_MINOR_DAC0     (16)   /*                                      */
#define  ML16P_MINOR_DAC1     (17)   /*                                      */
#define  ML16P_MINOR_DIGITAL  (18)   /*                                      */
#define  ML16P_MINOR_CT0      (19)   /*                                      */
#define  ML16P_MINOR_CT1      (20)   /*                                      */
#define  ML16P_MINOR_CT2      (21)   /*                                      */


/*  Number of jiffies to wait for conversion in hardware probe.              */
#define  ML16P_PROBE_TIME  (1)


/*  Read from the selected ADC here.                                         */
#define  ML16P_ADC_IN         (0x0)

/*  Write here to start ADC conversion.                                      */
#define  ML16P_ADC_START      (0x0)

/*  Write here to reset the End-of-ADC-Conversion flag.                      */
#define  ML16P_RESET_INT_REQ  (0x1)

/*  Command bytes are written here.                                          */
/*  See "driver/ml16p.h" for a description of Command bits.                  */
#define  ML16P_COMMAND        (0x2)


#define  ML16P_DIGITAL_IN   (0x3)
#define  ML16P_DIGITAL_OUT  (0x3)

#define  ML16P_DAC0         (0x4)
#define  ML16P_DAC1         (0x5)

#define  ML16P_CT0_READ     (0x8)
#define  ML16P_CT0_WRITE    (0x8)

#define  ML16P_CT1_READ     (0x9)
#define  ML16P_CT1_WRITE    (0x9)

#define  ML16P_CT2_READ     (0xa)
#define  ML16P_CT2_WRITE    (0xa)


/*  Write commands here to control the counter/timers.                       */
/*  See sys/ml16p.h for bit patterns and constants.                          */
#define  ML16P_CT_CONTROL  (0xb)

#define  ML16P_CT_LATCH_CMD  (0x00)
#define  ML16P_CT_W_LSB_MSB  (0x30)

#define  ML16P_CT_BIN  (0x00)
#define  ML16P_CT_BCD  (0x01)




/*  Macro definitions.                                                       */
#define  ML16P_ADC_START_CONVERSION(cardnum)  \
   outb(0x00, ml16p_card[cardnum].baseaddress + ML16P_ADC_START)

#define  ML16P_ADC_GET(cardnum)  \
   inb(ml16p_card[cardnum].baseaddress + ML16P_ADC_IN)

#define  ML16P_ADC_SELECT(adcnum, cardnum)  \
   outb(ml16p_card[cardnum].adcmode[minor], ml16p_card[cardnum].baseaddress + ML16P_COMMAND)

#define  ML16P_ADC_RESET_INT_REQ(cardnum)  \
   outb(0x00, ml16p_card[cardnum].baseaddress + ML16P_RESET_INT_REQ)

#define  ML16P_DAC_SET(val, dac_num, cardnum)  \
   outb(val, ml16p_card[cardnum].baseaddress + ML16P_DAC0 + dac_num)

#define  ML16P_DIGITAL_GET(cardnum)  \
   inb(ml16p_card[cardnum].baseaddress + ML16P_DIGITAL_IN)

#define  ML16P_DIGITAL_SET(val, cardnum)  \
   ml16p_card[cardnum].diglines = val;  \
   outb(ml16p_card[cardnum].diglines, ml16p_card[cardnum].baseaddress + ML16P_DIGITAL_OUT)

#define  ML16P_CT_GET(ct_num, cardnum)  \
   inb(ml16p_card[cardnum].baseaddress + ML16P_CT0_READ + ct_num)

#define  ML16P_CT_SET(val, ct_num, cardnum)  \
   outb(val, ml16p_card[cardnum].baseaddress + ML16P_CT0_WRITE + ct_num)


#define  ML16P_CARD_NUM(minor)       (minor / ML16P_MINOR_RANGE)
#define  ML16P_MINOR_START(cardnum)  (cardnum * ML16P_MINOR_RANGE)


/*  Miscellaneous constants.                                                 */
#define  ML16P_FALSE  (0)
#define  ML16P_TRUE   (!ML16P_FALSE)


#define  ML16P_DIFFERENTIAL  (0)
#define  ML16P_SINGLE_ENDED  (1)




struct ml16p_data {
   unsigned short baseaddress;   /*  Base address of card.                   */
   unsigned char irq;            /*  IRQ of card.                            */

   unsigned char adcmode[16];    /*  One status byte per ADC channel.        */ 
   unsigned char analog_type;    /*  Differential or Single-Ended?           */

   unsigned char diglines;       /*  Value currently on the digital lines.   */

   unsigned char ctmode[3];      /*  The Counter Control values of each c/t. */

   unsigned long read_in_use;    /*  Bitfield, minor numbers opened for      */
                                 /*  reading are 1's.                        */

   unsigned long write_in_use;   /*  Bitfield, minor numbers opened for      */
                                 /*  writing are 1's.                        */

   struct semaphore sem;         /*  Block use of card (waiting for ADC).    */
   struct wait_queue *wait_q;    /*  Sleep while waiting for interrupt.      */
};




#endif /* __ML16P_DRIVER_H */


