

#ifndef  __ML16P_H
#define  __ML16P_H




/*                                                                           */
/*  See the HTML users guide and the Model ML16-P Product Manual for more    */
/*  information.                                                             */
/*                                                                           */




/*                                                                           */
/*  This ioctl controls the format in which ADC-converted values are         */
/*  returned from the i/o card.  The two possible return formats are Offset  */
/*  (selected by this ioctl) and Twos Complement (selected by the ioctl      */
/*  ML16P_IO_TWOS_COMP).  Offset is the default return format.               */
/*  In the Offset format, 0x00 is the lowest possible value, 0x7f is         */
/*  midrange, and 0xff is the highest possible value.                        */
/*                                                                           */
#define  ML16P_IO_OFFSET  (0)
 



/*                                                                           */
/*  This ioctl controls the format in which ADC-converted values are         */
/*  returned from the i/o card.  The two possible return formats are Offset  */
/*  (selected by the ioctl ML16P_IO_OFFSET) and Twos Complement (selected    */
/*  by this ioctl).  Offset is the default return format.                    */
/*  In the Twos Complement format, a read 0x00 means 0 VDC.  0x7f is the     */
/*  highest possible voltage, 0x80 is the lowest possible voltage, and 0xff  */
/*  is the negative voltage of the greatest amplitude.                       */
/*                                                                           */
#define  ML16P_IO_TWOS_COMP  (1)




/*                                                                           */
/*  This ioctl helps control what input voltage range is permissible for a   */
/*  particular ADC by setting the input polarity to Unipolar.  The Unipolar  */
/*  input polarity specifies an input range of 0 VDC to +X VDC.  The value   */
/*  of X is determined by the "range size" ioctls below.                     */
/*                                                                           */
#define  ML16P_IO_UNIPOLAR  (2)




/*                                                                           */
/*  This ioctl helps control what input voltage range is permissible for a   */
/*  particular ADC by setting the input polarity to Bipolar.  The Bipolar    */
/*  input polarity specifies an input range of roughly -X VDC to +X VDC.     */
/*  The value for X is determined by the "range size" ioctls below.          */
/*                                                                           */
#define  ML16P_IO_BIPOLAR  (3)
 



/*                                                                           */
/*  This ioctl helps control what input voltage range is permissible for a   */
/*  particular ADC by setting the range size to Low.  Low range size         */
/*  implies a 0.255 VDC range, with 0.001 VDC resolution.                    */
/*                                                                           */
#define  ML16P_IO_LOW_RANGE  (4)




/*                                                                           */
/*  This ioctl helps control what input voltage range is permissible for a   */
/*  particular ADC by setting the range size to High.  Low range size        */
/*  implies a 9.961 VDC range, with 0.039 VDC resolution.                    */
/*                                                                           */
#define  ML16P_IO_HIGH_RANGE  (5)




/*                                                                           */
/*  This ioctl sets a particular bit of the digital output lines.            */
/*  Bits 6-4 of the argument specifies which bit to set.  Bit 0 specifies    */
/*  the desired new value for the bit.  For example, calling this ioctl      */
/*  with an argument of 0x41 would set bit 4 to 1.                           */
/*                                                                           */
#define  ML16P_IO_SETBIT  (6)




/*                                                                           */
/*  This ioctl sets the mode of a counter/timer.  The target mode is         */
/*  specified by the argument.  Valid modes are listed below.                */
/*                                                                           */
#define  ML16P_IO_CT_MODE  (7)




/*                                                                           */
/*  This ioctl returns information about the device it is called on.         */
/*  The argument is interpreted as a pointer to an unsigned char.  The       */
/*  calling program is responsible for making sure the pointer points        */
/*  somplace sensible.  The ioctl will write a byte to the given location.   */
/*  If called on an analog input line, the function writes the current       */
/*  configuration byte of the line (see bit values below).  This lets a      */
/*  user determine the current input range.                                  */
/*  If called on a counter/timer, the ioctl returns the current mode of the  */
/*  counter/timer (see values below).                                        */
/*  If called on any other device (the digital I/O lines or the DACs), the   */
/*  ioctl is a no-op, and the result is undefined.                           */
/*                                                                           */
#define  ML16P_IO_INFO  (8)


/*                                                                           */
/*  For the analog input lines, the byte written by the ML16P_IO_INFO ioctl  */
/*  should be interpreted as follows.                                        */
/*  +--------------+-----+-----+-----+-----+-----+-----+-----+-----+         */
/*  | bit position | D7  | D6  | D5  | D4  | D3  | D2  | D1  | D0  |         */
/*  +--------------+-----+-----+-----+-----+-----+-----+-----+-----+         */
/*  | information  | ATO | FRM | B/U | RGE | MA3 | MA2 | MA1 | MA0 |         */
/*  +--------------+-----+-----+-----+-----+-----+-----+-----+-----+         */
/*  ATO 0: Automatic conversion start disabled.                              */
/*      1: Automatic conversion start enabled.                               */
/*  FRM 0: Offset binary return format.                                      */
/*      1: Twos Complement return format.                                    */
/*  B/U 0: Unipolar input polarity.                                          */
/*      1: Bipolar input polarity.                                           */
/*  RGE 0: Low input range size.                                             */
/*      1: High input range size.                                            */
/*  MA{3-0} are the Analog Input Channed Address.                            */
/*                                                                           */
/*  Only FRM, B/U, and RGE are user configurable and user intelligible.  ATO */
/*  is used by the driver during read() operations and cannot (and certainly */
/*  should not) be changed by the user.  MA{3-0} are simply the address of   */
/*  the analog input line that the ioctl was called on.                      */
/*                                                                           */
#define  ML16P_ATO  (0x80)
#define  ML16P_FRM  (0x40)
#define  ML16P_BU   (0x20)
#define  ML16P_RGE  (0x10)


/*  These are the constants used to indicate the mode of the counter/timers. */
#define ML16P_CT_MODE_MASK 0x0e

#define  ML16P_CT_MODE_0  (0x00)
#define  ML16P_CT_MODE_1  (0x02)
#define  ML16P_CT_MODE_2  (0x04)
#define  ML16P_CT_MODE_3  (0x06)
#define  ML16P_CT_MODE_4  (0x08)
#define  ML16P_CT_MODE_5  (0x0a)

#define  ML16P_CT_PULSE_ON_TERMINAL_COUNT    ML16P_CT_MODE_0
#define  ML16P_CT_RETRIGGERABLE_ONE_SHOT     ML16P_CT_MODE_1
#define  ML16P_CT_RATE_GENERATOR             ML16P_CT_MODE_2
#define  ML16P_CT_SQUARE_WAVE_GENERATOR      ML16P_CT_MODE_3
#define  ML16P_CT_SOFTWARE_TRIGGERED_STROBE  ML16P_CT_MODE_4
#define  ML16P_CT_HARDWARE_TRIGGERED_STROBE  ML16P_CT_MODE_5




/*  These constants correspond to the limits of the ADCs.                    */
#define  ML16P_ADC_UNI_LO_MAX  (+0.255)
#define  ML16P_ADC_UNI_LO_MIN  (+0.000)

#define  ML16P_ADC_UNI_HI_MAX  (+9.961)
#define  ML16P_ADC_UNI_HI_MIN  (+0.000)

#define  ML16P_ADC_BI_LO_MAX  (+0.127)
#define  ML16P_ADC_BI_LO_MIN  (-0.128)

#define  ML16P_ADC_BI_HI_MAX  (+4.961)
#define  ML16P_ADC_BI_HI_MIN  (-5.000)


/*  These constants correspond to the limits of the DACs.                    */
#define  ML16P_DAC_MAX  (+9.922)
#define  ML16P_DAC_MIN  (-10.000)




/*  The ioctls below this line are for debugging purposes ONLY!              */
/*  Use at your own risk!                                                    */
#define  ML16P_DEBUG_IO_EN_AUTOCON   (9)   /*  Sets an ADC to autoconvert.   */
#define  ML16P_DEBUG_IO_DIS_AUTOCON  (10)  /*  Disable autoconv for an ADC.  */
#define  ML16P_DEBUG_IO_RIR          (11)  /*  Force a RIR.                  */
#define  ML16P_DEBUG_IO_ADC_START    (12)  /*  Force ADC conversion start.   */
#define  ML16P_DEBUG_IO_ADC_READ     (13)  /*  Force a read from the ADC.    */
#define  ML16P_DEBUG_IO_ADC_SELECT   (14)  /*  Explicitly select an ADC.     */




#endif  /*  not __ML16P_H  */


