

/*
 *  count.c
 *  Copyright 1995 1996, Sebastian Kuzminsky <kuzminsk@cs.colorado.edu>.
 *  See the file "doc/COPYING" for detailed copyright information.
 *
 *
 *     This program is a command-line utility intended for distribution and use
 *  with the Linux ML16-P driver.  It can retrieve the current count of a
 *  counter/timer and report it to the user.  It can also set the count to a
 *  specified value.
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <strings.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <unistd.h>
#include <sys/ioctl.h>


#include "../driver/ml16p.h"

#include "support.h"




void printhelp(void);




int main(int argc, char **argv) {
   unsigned short val;
   long i;

   int devicefile = -1;


/*  Parse command line, looking for options.                                 */
   for(i = 1; i < argc; i ++) {
      if((strcmp(argv[i], "-h") == 0) || (strcmp(argv[i], "--help") == 0)) {
         printhelp();

         exit(0);
      }

      if((strcmp(argv[i], "-v") == 0) || (strcmp(argv[i], "--version") == 0)) {
         printf("count - " VERSION "\n");

         exit(0);
      }
   }

   if((argc != 2) && (argc != 3)) {
      printhelp();

      exit(1);
   }


/*  If there is only one argument, then that must be the device file to use. */
   if(argc == 2) {
      if((devicefile = open(argv[1], O_RDONLY)) == -1) {
	 fprintf(stderr, "cannot open %s for read\n", argv[1]);
	 perror(argv[1]);
	 exit(1);
      }

/*  Read the 16-bit count into an unsigned short int.                        */
      if(read(devicefile, &val, 2) != 2) {
         fprintf(stderr, "read error\n");
	 perror(argv[1]);
         exit(1);
      }

/*  Print it.                                                                */
      printf("0x%04x\n", val);

      close(devicefile);
      exit(0);
   }


/*  If we get here, then two arguments were given, and we should set the     */
/*  count.                                                                   */
   if((devicefile = open(argv[1], O_WRONLY)) == -1) {
      fprintf(stderr, "cannot open %s for write\n", argv[1]);
      perror(argv[0]);
      exit(1);
   }

/*  Convert the new count from a string (on command line) to an unsigned     */
/*  short integer.  Note that what we really want is a 16 bit quantity, and  */
/*  shorts are 16 bits on x86 CPUs.                                          */
   i = strtol(argv[2], NULL, 0);
   if((i < 0) || (i > 65535)) {
      fprintf(stderr, "counts in the range 0 to 65535 can be written\n");
      exit(1);
   }
   val = (unsigned short)i;

/*  Write the value to the counter/timer.                                    */
   if(write(devicefile, &val, 2) != 2) {
      fprintf(stderr, "write error\n");
      perror(argv[1]);
      exit(1);
   }
   printf("0x%04x\n", val);

   close(devicefile);
   exit(0);
}




void printhelp(void) {
   printf("Usage:  count [OPTIONS]\n");
   printf("        count DEVICE [count]\n");
   printf("Gets or sets the count of a counter/timer on an ML16-P I/O card.\n");
   printf("\n");
   printf("   -h,  --help     print this help and exit\n");
   printf("   -v,  --version  print version information and exit\n");
   printf("\n");
   printf("DEVICE is the counter/timer device to access (for example, /dev/ml16pa-ct0)\n");
   printf("value is the optional count to write to that counter/timer\n");
   printf("\n");
   printf("If no count is given on the command line, then the current count is read\n");
   printf("from the given device and reported.  If a count is supplied, then it is\n");
   printf("written as the new count.\n");
}


