

/*
 *  ctmode.c
 *  Copyright 1996, Sebastian Kuzminsky <kuzminsk@cs.colorado.edu>.
 *  See the file "doc/COPYING" for detailed copyright information.
 *
 *
 *     This program is a command-line utility intended for distribution and use
 *  with the Linux ML16-P driver.  It can retrieve the current configuration of
 *  a Counter/Timer and report it to the user.  It can also change the current
 *  configuration of a Counter/Timer.
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>


#include "../driver/ml16p.h"

#include "support.h"




void printhelp(void);




int main(int argc, char **argv) {
   unsigned char val;

   int devicefile = -1;

   int i;


/*  Parse command line, looking for options.                                 */
   for(i = 1; i < argc; i ++) {
      if((strcmp(argv[i], "-h") == 0) || (strcmp(argv[i], "--help") == 0)) {
	 printhelp();

	 exit(0);
      }

      if((strcmp(argv[i], "-v") == 0) || (strcmp(argv[i], "--version") == 0)) {
	 printf("range - " VERSION "\n");

	 exit(0);
      }
   }

/*  Incorrect number of command-line arguments?                              */
   if((argc != 3) && (argc != 2)) { 
      printhelp();
      exit(1);
   }


   if((devicefile = open(argv[1], O_RDONLY)) == -1) {
      fprintf(stderr, "cannot open %s for read\n", argv[1]);
      perror(argv[1]);
      exit(1);
   }

   if(argc == 3) {
/*  First set the mode as per the command line.                              */
      if(ioctl(devicefile, ML16P_IO_CT_MODE, atol(argv[2])) == -1) {
	 fprintf(stderr, "cannot switch to mode %ld\n", atol(argv[2]));
	 perror(argv[1]);
	 exit(1);
      }
   }


/*  Next we get the info back from the counter/timer...                      */
   if(ioctl(devicefile, ML16P_IO_INFO, &val) == -1) {
      fprintf(stderr, "cannot execute INFO ioctl\n");
      perror(argv[1]);
      exit(1);
   }

   switch(val & ML16P_CT_MODE_MASK) {
      case ML16P_CT_MODE_0:
         printf("Mode 0, Pulse on Terminal Count\n");
         break;

      case ML16P_CT_MODE_1:
         printf("Mode 1, Retriggerable One-Shot\n");
         break;

      case ML16P_CT_MODE_2:
         printf("Mode 2, Rate Generator\n");
         break;

      case ML16P_CT_MODE_3:
         printf("Mode 3, Square Wave Generator\n");
         break;

      case ML16P_CT_MODE_4:
         printf("Mode 4, Software Triggered Strobe\n");
         break;

      case ML16P_CT_MODE_5:
         printf("Mode 5, Hardware Triggered Strobe\n");
         break;

      default:
         printf("unknown mode (this is bad)\n");
         exit(1);
         break;

   }

   exit(0);
}




void printhelp(void) {
   printf("Usage:  ctmode [OPTIONS]\n");
   printf("        ctmode DEVICE [MODE]\n");
   printf("Gets or sets the input range of a counter/timer on an ML16-P I/O card.\n");
   printf("\n");
   printf("   -h,  --help     print this help and exit\n");
   printf("   -v,  --version  print version information and exit\n");
   printf("\n");
   printf("DEVICE is the Counter/Timer device to access (for example, /dev/ml16pa-ct0)\n");
   printf("The valid MODEs are 0 - 5.\n");
   printf("\n");
   printf("When a MODE is given, then the Counter/Timer is configured as specified and the\n");
   printf("new configuration is printed out.");
   printf("\n");
   printf("If no MODE is given, the current configuration is printed out\n");
   printf("\n");
}


