

/*
 *  setbit.c
 *  Copyright 1995 1996, Sebastian Kuzminsky <kuzminsk@cs.colorado.edu>.
 *  See the file "doc/COPYING" for detailed copyright information.
 *
 *
 *     This program is a command-line utility intended for distribution and use
 *  with the Linux ML16-P driver.  It is used to set the value of individual
 *  bits of the digital ouput lines.
 *
 */


#include <stdio.h>
#include <stdlib.h>

#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>


#include "../driver/ml16p.h"

#include "support.h"




void printhelp(void);




int main(int argc, char *argv[]) {
   int devfile;

   unsigned long val;

   int i;


/*  Parse command line, looking for options.                                 */
   for(i = 1; i < argc; i ++) {
      if((strcmp(argv[i], "-h") == 0) || (strcmp(argv[i], "--help") == 0)) {
	 printhelp();

	 exit(0);
      }

      if((strcmp(argv[i], "-v") == 0) || (strcmp(argv[i], "--version") == 0)) {
	 printf("setbit - " VERSION "\n");

	 exit(0);
      }
   }

/*  Incorrect nuber of command line arguments or help requested?             */
   if(argc != 4) {
      printhelp();
      exit(1);
   }

/*  Setting bits is purely a writing operation, so we open the requested     */
/*  device write-only.                                                       */
   if((devfile = open(argv[1], O_WRONLY)) == -1) {
      fprintf(stderr, "cannot open %s for write\n", argv[1]);
      perror(argv[1]);
      exit(1);
   }

/*  Figure out what bit to set and what to set it to.                        */
/*  Note the the packing of the information into an unsigned long:           */
/*     -  The number of the bit to chage is stored in the second to least    */
/*        significant nibble.                                                */
/*     -  The new value of the bit is stored in the least significant bit.   */
   val = strtoul(argv[2], NULL, 0) << 4;
   val |= strtoul(argv[3], NULL, 0);

/*  Now execute the SETBIT ioctl to actually set the bit.                    */
   if(ioctl(devfile, ML16P_IO_SETBIT, val) == -1) {
      fprintf(stderr, "cannot execute SETBIT ioctl\n");
      perror("SETBIT ioctl");
      close(devfile);
      exit(1);
   }

/*  Clean up and go home.                                                    */
   close(devfile);

   exit(0);
}




void printhelp(void) {
   printf("Usage:  setbit [OPTIONS]\n");
   printf("        setbit DEVICE bit value\n");
   printf("Sets a particular bit of the digital output lines of an ML16-P I/O card.\n");
   printf("\n");
   printf("   -h,  --help     print this help and exit\n");
   printf("   -v,  --version  print version information and exit\n");
   printf("\n");
   printf("DEVICE is digital device to access (for example, /dev/ml16pa-digital)\n");
   printf("bit must be in the range 0-7\n");
   printf("value must be 0 or 1\n");
   printf("\n");
}


