

/*  sine.c
 *  Copyright 1995 1996, Sebastian Kuzminsky <kuzminsk@cs.colorado.edu>.
 *  See the file "doc/COPYING" for detailed copyright information.
 *
 *  This program is a command-line utility intended for distribution and use
 *  with the Linux ML16-P driver.  It is used to output a sine wave from one of
 *  the DACs.
 */


#include <stdio.h>
#include <stdlib.h>

#include <math.h>

#include <fcntl.h>
#include <unistd.h>


#include "../driver/ml16p.h"

#include "support.h"




void print_help(void);




int main(int argc, char *argv[]) {
   double freq = 1.0, amp = 1.0;

   char *dev_name = "/dev/ml16pa-dac0";
   int dev_fd;

   int n;

   double t;
   double volts;
   unsigned char out_val;


   for(n = 1; n < argc; n ++) {
      if((strcmp(argv[n], "-f") == 0) || (strcmp(argv[n], "--frequency") == 0)) {
         n ++;
         freq = strtod(argv[n], NULL);
         continue;
      }

      if((strcmp(argv[n], "-a") == 0) || (strcmp(argv[n], "--amplitude") == 0)) {
         n ++;
         amp = strtod(argv[n], NULL);

         if((amp < 0.0) || (amp > ML16P_DAC_MAX)) {
            fprintf(stderr, "valid amplitudes are 0 <= amp <= %f\n", ML16P_DAC_MAX);
            exit(1);
         }

         continue;
      }

      if((strcmp(argv[n], "-d") == 0) || (strcmp(argv[n], "--device") == 0)) {
         n ++;
         dev_name = argv[n];
         continue;
      }

      if((strcmp(argv[n], "-v") == 0) || (strcmp(argv[n], "--version") == 0)) {
         printf("sine - " VERSION "\n");
         exit(0);
      }

      print_help();
      exit(0);
   }

   dev_fd = open(dev_name, O_WRONLY);
   if(dev_fd == -1) {
      fprintf(stderr, "error on open()\n");
      perror(dev_name);
      exit(1);
   }

   do {
      for(t = 0; t < (1.0 / freq); t += (1.0 / 60000.0)) {
         volts = amp * sin(t * ((2 * M_PI) / (1.0 / freq)));

         out_val = (unsigned char)(255.0 * ((volts - ML16P_DAC_MIN) / (ML16P_DAC_MAX - ML16P_DAC_MIN)));
	 if(write(dev_fd, &out_val, 1) != 1) {
	    fprintf(stderr, "error on write()\n");
	    perror(argv[1]);
	    exit(1);
	 }


      }
   } while(1);

   exit(0);
}




void print_help(void) {
   fprintf(stderr, "Usage:  sine [OPTIONS] [DEVICE FILE]\n");
   fprintf(stderr, "\n");
   fprintf(stderr, "   -f, --frequency freq     freq is the output frequency in Hertz.  0.0 to a\n");
   fprintf(stderr, "                            few thousand Hz should be doable.  Default is 1.0.\n");
   fprintf(stderr, "   -a, --amplitude amp      amp is the output amplitude, in Volts.  The sine\n");
   fprintf(stderr, "                            wave is centered around 0.0 V.  The maximum\n");
   fprintf(stderr, "                            amplitude is 9.922 V.  Default is 1.0 V.\n");
   fprintf(stderr, "   -d, --device device_file Specify which device file to use.  The default is\n");
   fprintf(stderr, "                            /dev/ml16pa-dac0.\n");
   fprintf(stderr, "   -h, --help               Print this help and exit.\n");
   fprintf(stderr, "\n");
   fprintf(stderr, "\n");
}


